﻿using ProjectMotorboat.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMotorboat.Drownings;

public class DrawningBoat
{
    public EntityBoat? EntityBoat { get; protected set; }

    private int? _pictureWidth;

    private int? _pictureHeight;

    protected int? _startPosX;

    protected int? _startPosY;
    private readonly int _drawningBoatWidth = 160;

    private readonly int _drawningBoatHeight = 60;

    public int? GetPosX => _startPosX;

    
    public int? GetPosY => _startPosY;

    
    public int GetWidth => _drawningBoatWidth;

   
    public int GetHeight => _drawningBoatHeight;
    private DrawningBoat()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    public DrawningBoat(int speed, double weight, Color bodyColor) : this()
    {
        EntityBoat = new EntityBoat(speed, weight, bodyColor);
       
    }
    protected DrawningBoat(int drawningBoatWidth, int drawningBoatHeight) : this() 
    {
        _drawningBoatWidth = drawningBoatWidth;
        _drawningBoatHeight = drawningBoatHeight;
        
    }
    public bool SetPictureSize(int width, int height)
    {
        if (_drawningBoatWidth < width && _drawningBoatHeight < height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX.HasValue && _startPosY.HasValue)
            {
                SetPosition(_startPosX.Value, _startPosY.Value);
            }

            return true;
        }

        return false;
    }

    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        if (x + _drawningBoatWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningBoatWidth;
        }
        else if (x < 0)
        {
            _startPosX = 0;
        }
        else
        {
            _startPosX = x;
        }
        if (y + _drawningBoatHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningBoatHeight;
        }
        else if (y < 0)
        {
            _startPosY = 0;
        }
        else
        {
            _startPosY = y;
        }
    }
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityBoat == null || !_startPosX.HasValue ||
        !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityBoat.Step > 0)
                {
                    _startPosX -= (int)EntityBoat.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityBoat.Step > 0)
                {
                    _startPosY -= (int)EntityBoat.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityBoat.Step + _drawningBoatWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityBoat.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityBoat.Step + _drawningBoatHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityBoat.Step;
                }
                return true;
            default:
                return false;
        }
    }


    public virtual void DrawTransport(Graphics g)
    {
        if (EntityBoat == null || !_startPosX.HasValue || !_startPosY.HasValue)
        { return; }
        Pen pen = new(Color.Black);
        Brush mainBrush = new SolidBrush(EntityBoat.BodyColor);

        // корпус
        Point[] hull = new Point[]
        {
                new Point(_startPosX.Value + 5, _startPosY.Value + 0),
                new Point(_startPosX.Value + 120, _startPosY.Value + 0),
                new Point(_startPosX.Value + 160, _startPosY.Value + 35),
                new Point(_startPosX.Value + 120, _startPosY.Value + 70),
                new Point(_startPosX.Value + 5, _startPosY.Value + 70),
        };
        g.FillPolygon(mainBrush, hull);
        g.DrawPolygon(pen, hull);

        // основная часть
        Brush blockBrush = new SolidBrush(EntityBoat.BodyColor);
        g.FillRectangle(blockBrush, _startPosX.Value + 20, _startPosY.Value + 15, 80, 40);
        g.DrawRectangle(pen, _startPosX.Value + 20, _startPosY.Value + 15, 80, 40);
}
}
